<?php

namespace Drupal\drupalmoduleupgrader\Converter;

/**
 * Base interface for all converters. Different converter plugin types might
 * extend this interface, but ultimately all converters must implement this.
 */
interface ConverterInterface {

  /**
   * Checks if this plugin can be executed, given the current module context.
   * This is called before analyze() or convert(), and if it returns a falsy
   * value, neither will be called.
   *
   * @return boolean
   */
  public function isExecutable();

  /**
   * Scans for problems that needs fixing, and reports them.
   */
  public function analyze();

  /**
   * Performs the required conversion(s).
   */
  public function convert();

}
