<?php

namespace Drupal\drupalmoduleupgrader\Converter\FunctionReplacement;

use Drupal\drupalmoduleupgrader\Converter\ConverterBase;
use Pharborist\Filter;
use Pharborist\FunctionCallNode;

abstract class FunctionCallModifier extends ConverterBase {

  /**
   * The function calls to be processed, keyed by the file in which
   * they occur.
   *
   * @var \Pharborist\NodeCollection[]
   */
  protected $functionCalls;

  /**
   * Constructs a FunctionCallModifier.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    // The affected function can either be the plugin ID, or explicitly defined in the plugin
    // definition (useful for derivatives).
    $function = $plugin_definition['function'] ?: $this->getPluginId();

    $files = $this->db
      ->select('dmu_index', 'i')
      ->distinct()
      ->fields('i', ['file'])
      ->condition('module', $this->module->getMachineName())
      ->condition('type', 'function_call')
      ->condition('name', $function)
      ->execute()
      ->fetchCol();

    foreach ($files as $file) {
      $this->functionCalls[$file] = $this->module
        ->getFile($file)
        ->find(Filter::isFunctionCall($function))
        ->filter([ $this, 'canModify' ]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function analyze() {
    $definition = $this->pluginDefinition;

    if (isset($definition['message'])) {
      parent::analyze();

      foreach ($this->functionCalls as $file => $calls) {
        foreach ($calls as $call) {
          $this->module->setViolation($definition['issue_type'], $file, $call->getSourcePosition());
        }
      }
    }
  }

  /**
   * Returns if a function call can be modified.
   *
   * @param \Pharborist\FunctionCallNode $call
   *  The function call to test.
   *
   * @return boolean
   */
  public function canModify(FunctionCallNode $call) {
    return TRUE;
  }

}
