<?php

/**
 * @file
 * Contains Drupal\drupalmoduleupgrader\Converter\HookConverterBase.
 */

namespace Drupal\drupalmoduleupgrader\Converter;

use Drupal\drupalmoduleupgrader\Utility\FunctionAnalyzerTrait;
use Pharborist\DocCommentNode;

/**
 * Base class to convert hooks.
 */
abstract class HookConverterBase extends ConverterBase {

  use FunctionAnalyzerTrait;

  /**
   * The hook implementation.
   *
   * @var \Pharborist\FunctionDeclarationNode
   */
  protected $hook;

  /**
   * Constructs a HookConverterBase object.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    if ($this->module->defines('hook', $plugin_definition['hook'])) {
      $this->hook = $this->module->getHook($plugin_definition['hook']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isExecutable() {
    if (isset($this->hook)) {
      if ($this->containsLogic($this->hook)) {
        drush_log('Cannot convert a hook implementation that contains logic.', 'error');
        return FALSE;
      }
      else {
        return TRUE;
      }
    }
    else {
      drush_log("No implementation of hook_" . $this->pluginDefinition['hook'] . "() found.", 'notice');
      return FALSE;
    }
  }

  /**
   * Sets a @FIXME notice above the hook implementation. The notice's text is
   * pulled from the plugin definition.
   */
  protected function addFixMe() {
    if ($this->pluginDefinition['fixme']) {
      $comment = $this->hook->getDocComment();
      $comment_text = $comment ? $comment->getCommentText() : 'Implements hook_' . $this->pluginDefinition['hook'] . '().';
      $this->hook->setDocComment(DocCommentNode::create("{$comment_text}\n\n@FIXME\n" . $this->pluginDefinition['fixme']));
    }
  }

}
