<?php

/**
 * @file
 * Contains \Drupal\drupalmoduleupgrader\Converter\Routing\Drupal7\ParameterMap.
 */

namespace Drupal\drupalmoduleupgrader\Converter\Routing\Drupal7;

use Drupal\drupalmoduleupgrader\Converter\Routing\ParameterBindingInterface;
use Drupal\drupalmoduleupgrader\Converter\Routing\ParameterMapBase;
use Drupal\drupalmoduleupgrader\Utility\Path\Drupal7\PathComponent;
use Drupal\drupalmoduleupgrader\Utility\Path\Drupal8\PathComponent as PathComponent8x;
use Drupal\drupalmoduleupgrader\Utility\Path\PathUtilityInterface;
use Symfony\Component\Routing\Route as Drupal8Route;

/**
 * Represents a set of parameter bindings for a particular path, callback,
 * and set of arguments in a Drupal 7 route.
 */
class ParameterMap extends ParameterMapBase {

  /**
   * @var integer
   */
  private $_length = 0;

  /**
   * {@inheritdoc}
   */
  public function __construct(PathUtilityInterface $path, array $parameters, array $arguments = []) {
    parent::__construct($path, $parameters);

    $this->_length = sizeof($path);

    while ($parameters) {
      $argument = $arguments ? array_shift($arguments) : ParameterBinding::NO_ARGUMENT;
      $this->addBinding(new ParameterBinding($path, array_shift($parameters), $argument));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function addBinding(ParameterBindingInterface $binding) {
    $value = $binding->getValue();

    // The binding will return a PathComponent if it expects to be physically
    // represented in the path, whether or not it already is.
    if ($value instanceof PathComponent) {
      if ($binding->inPath()) {
        $key = $binding->getArgument();
      }
      else {
        $key = $this->path->indexOf($value);
        if ($key === FALSE) {
          $key = $this->_length++;
        }

        if (isset($this->bindings[$key])) {
          $binding->enslave($this->bindings[$key]);
        }
      }
    }
    else {
      $key = $binding->getParameter()->getName();
    }

    $this->bindings[$key] = $binding;
  }

  /**
   * {@inheritdoc}
   */
  public function apply(Drupal8Route $route) {
    foreach ($this->bindings as $key => $binding) {
      $parameter = $binding->getParameter();

      if (is_integer($key)) {
        $this->path[$key] = new PathComponent8x('{' . $parameter->getName() . '}');

        if ($parameter->isOptional()) {
          $value = eval('return ' . $parameter->getValue() . ';');
          $route->setDefault($parameter->getName(), $value);
        }

        $binding->conform();
      }
      elseif ($binding->hasArgument()) {
        $route->setDefault($parameter->getName(), $binding->getValue());
      }
    }
    $route->setPath($this->path->__toString());
  }

}
