<?php

/**
 * @file
 * Contains \Drupal\drupalmoduleupgrader\Converter\Routing\Drupal8\ParameterMap.
 */

namespace Drupal\drupalmoduleupgrader\Converter\Routing\Drupal8;

use Drupal\drupalmoduleupgrader\Converter\Routing\ParameterBindingInterface;
use Drupal\drupalmoduleupgrader\Converter\Routing\ParameterMapBase;
use Drupal\drupalmoduleupgrader\Utility\Path\PathUtilityInterface;
use Symfony\Component\Routing\Route;

/**
 * Represents a set of path-based parameter bindings in a Drupal 8 route.
 */
class ParameterMap extends ParameterMapBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(PathUtilityInterface $path, array $parameters) {
    parent::__construct($path, $parameters);

    while ($parameters) {
      $this->addBinding(new ParameterBinding($path, array_shift($parameters));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function add(ParameterBindingInterface $binding) {
    if ($binding->inPath()) {
      $key = $binding->getParameter()->getName();

      if (isset($this->bindings[$key])) {
        $binding->enslave($this->bindings[$key]);
      }

      $this->bindings[$key] = $binding;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function apply(Route $route) {
    foreach ($this->bindings as $key => $binding) {
      $binding->conform();
      $route->setDefault($key, eval('return ' . $binding->getParameter()->getValue() . ';'));
    }
  }

}
