<?php

/**
 * @file
 * Contains Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide\FunctionReplacement.
 */

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide;

use Drupal\drupalmoduleupgrader\Converter\ConverterBase;

/**
 * @Converter(
 *  id = "FunctionReplacement",
 *  description = @Translation("Rewrites function calls that have changed in Drupal 8."),
 *  delegator = "plugin.manager.drupalmoduleupgrader.function"
 * )
 */
class FunctionReplacement extends ConverterBase {

  /**
   * Every function call in the module being converted.
   *
   * @var array
   */
  private $functions = [];

  /**
   * Plugin IDs for every available and applicable delegate.
   *
   * @var string[]
   */
  private $delegateIDs = [];

  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->functions = $this->module->getObjectNames('function_call');

    // Figure out which delegates we will need so that they can be instantiated
    // on demand.
    foreach ($this->delegator->getDefinitions() as $plugin_id => $definition) {
      $function = isset($definition['function']) ? $definition['function'] : $plugin_id;
      if (in_array($function, $this->functions)) {
        $this->delegateIDs[] = $plugin_id;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function analyze() {
    foreach ($this->delegateIDs as $plugin_id) {
      $this->getDelegate($plugin_id)->analyze();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function convert() {
    foreach ($this->delegateIDs as $plugin_id) {
      $this->getDelegate($plugin_id)->convert();
    }
    $this->module->save();
  }

}
