<?php

/**
 * @file
 * Contains Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide\HookBlockInfo.
 */

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide;

use Drupal\drupalmoduleupgrader\Converter\HookConverterBase;
use Drupal\drupalmoduleupgrader\Utility\StringTransformTrait;

/**
 * @Converter(
 *  id = "HookBlockInfo",
 *  description = @Translation("Converts Drupal 7's hook_block_info() to plugins."),
 *  documentation = {
 *    {
 *      "url" = "https://api.drupal.org/api/drupal/core%21modules%21block%21block.api.php/group/block_api/8",
 *      "title" = @Translation("Drupal 8 Block API documentation")
 *    }
 *  },
 *  message = @Translation("Blocks are now plugins in the <code>MODULE\Plugin\Block</code> namespace."),
 *  hook = "block_info"
 * )
 */
class HookBlockInfo extends HookConverterBase {

  use StringTransformTrait;

  /**
   * {@inheritdoc}
   */
  public function convert() {
    $blocks = $this->module->executeHook('block_info');

    foreach ($blocks as $id => $info) {
      // Render the block plugin's shell.
      $render = [
        '#theme' => 'dmu_block',
        '#module' => $this->module->getMachineName(),
        '#class' => $this->toTitleCase($id),
        '#block_id' => $id,
        '#block_label' => $info['info'],
      ];

      $this->module->write('src/Plugin/Block/' . $render['#class'] . '.php', "<?php\n\n" . render($render));
      drush_log("Processed {$id} block.", 'notice');
    }

    // @todo Delete the hook implementation when everything in it has been fully converted.
    // $this->hook->remove();
    $count = \Drupal::translation()->formatPlural(sizeof($blocks), '1 block', '@count blocks');
    drush_log("Converted {$count} to plugins.", 'success');
  }

}
