<?php

/**
 * @file
 * Contains Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide\HookExit.
 */

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide;

use Drupal\drupalmoduleupgrader\Converter\HookConverterBase;
use Drupal\drupalmoduleupgrader\Utility\ClassGeneratorTrait;

/**
 * @Converter(
 *  id = "HookExit",
 *  description = @Translation("Converts Drupal 7's hook_exit() to an EventSubscriber."),
 *  documentation = {
 *    {
 *      "url" = "https://www.drupal.org/node/1911186",
 *      "title" = @Translation("<code>hook_exit()</code> replaced by event subscriber")
 *    }
 *  },
 *  message = @Translation("<code>hook_exit()</code> has been removed."),
 *  hook = "exit"
 * )
 */
class HookExit extends HookConverterBase {

  use ClassGeneratorTrait;

  /**
   * {@inheritdoc}
   */
  public function convert() {
    $service = [
      'class' => $this->module->getNS() . '\\EventSubscriber\\ExitSubscriber',
      'tags' => [
        [ 'name' => 'event_subscriber' ],
      ],
    ];
    $this->module->addService('exit_subscriber', $service);

    $subscriber = $this->generate();
    $this->addMethod($this->module->getHook('exit'), $subscriber, 'onEvent');

    $this->module->write('src/EventSubscriber/ExitSubscriber.php', $subscriber->parents()[0]);
    drush_log('Generated KernelEvents::TERMINATE event subscriber.', 'success');
  }
  
  /**
   * {@inheritdoc}
   */
  public function generate() {
    $render = [
      '#theme' => 'dmu_event_subscriber',
      '#module' => $this->module->getMachineName(),
      '#class' => 'ExitSubscriber',
      '#event' => 'KernelEvents::TERMINATE',
    ];

    return $this->parse($render);
  }

}
