<?php

/**
 * @file
 * Contains Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide\HookFormAlter.
 */

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide;

use Drupal\drupalmoduleupgrader\Converter\ConverterBase;
use Pharborist\Filter;

/**
 * @Converter(
 *  id = "HookFormAlter",
 *  description = @Translation("Corrects hook_form_alter() function signatures."),
 *  documentation = {
 *    {
 *      "url" = "https://api.drupal.org/api/drupal/core%21modules%21system%21system.api.php/function/hook_form_alter/8",
 *      "title" = @Translation("<code>hook_form_alter()<code> documentation")
 *    }
 *  },
 *  message = @Translation("The signature of hook_form_alter() has changed in Drupal 8.")
 * )
 */
class HookFormAlter extends ConverterBase {

  /**
   * {@inheritdoc}
   */
  public function analyze() {
    // @todo
  }

  /**
   * {@inheritdoc}
   */
  public function convert() {
    $alter_hooks = $this->db
      ->select('dmu_index', 'i')
      ->fields('i', ['name'])
      ->condition('type', 'function')
      ->condition(db_or()
        ->condition('name', $this->module->getMachineName() . '_form_alter')
        ->condition('name', $this->module->getMachineName() . '_form_%_alter', 'LIKE')
      )
      ->execute()
      ->fetchCol();

    foreach ($alter_hooks as $alter_hook) {
      $parameters = $this->module
        ->getFileContaining('function', $alter_hook)
        ->children(Filter::isFunction($alter_hook))
        ->get(0)
        ->getParameters();

      if (sizeof($parameters) > 1) {
        $parameters[1]->setTypeHint('\Drupal\Core\Form\FormStateInterface');
      }
    }
  }

}
