<?php

/**
 * @file
 * Contains Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide\HookInit.
 */

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide;

use Drupal\drupalmoduleupgrader\Converter\HookConverterBase;
use Drupal\drupalmoduleupgrader\Utility\ClassGeneratorTrait;

/**
 * @Converter(
 *  id = "HookInit",
 *  description = @Translation("Converts Drupal 7's hook_init() to an EventSubscriber."),
 *  documentation = {
 *    {
 *      "url" = "https://www.drupal.org/node/2013014",
 *      "title" = @Translation("<code>hook_init()</code> removed")
 *    }
 *  },
 *  message = @Translation("hook_init() has been removed in Drupal 8 and replaced by an event subscription system."),
 *  hook = "init"
 * )
 */
class HookInit extends HookConverterBase {

  use ClassGeneratorTrait;

  /**
   * {@inheritdoc}
   */
  public function convert() {
    $service = [
      'class' => $this->module->getNS() . '\\EventSubscriber\\InitSubscriber',
      'tags' => [
        [ 'name' => 'event_subscriber' ],
      ],
    ];
    $this->module->addService('init_subscriber', $service);
    
    $subscriber = $this->generate();
    $this->addMethod($this->module->getHook('init'), $subscriber, 'onEvent');

    $this->module->write('src/EventSubscriber/InitSubscriber.php', $subscriber->parents()[0]);
    drush_log('Generated KernelEvents::REQUEST event subscriber.', 'success');
  }
  
  /**
   * {@inheritdoc}
   */
  public function generate() {
    $render = [
      '#theme' => 'dmu_event_subscriber',
      '#module' => $this->module->getMachineName(),
      '#class' => 'InitSubscriber',
      '#event' => 'KernelEvents::REQUEST',
    ];

    return $this->parse($render);
  }

}
