<?php

/**
 * @file
 * Contains Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide\HookURLOutboundAlter.
 */

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide;

use Drupal\drupalmoduleupgrader\Converter\HookConverterBase;
use Drupal\drupalmoduleupgrader\Utility\ClassGeneratorTrait;

/**
 * @Converter(
 *  id = "HookURLOutboundAlter",
 *  description = @Translation("Converts hook_url_outbound_alter() to a service."),
 *  documentation = {
 *    {
 *      "url" = "https://www.drupal.org/node/2238759",
 *      "title" = @Translation("<code>hook_url_outbound_alter()</code> replaced by <code>OutboundPathProcessorInterface</code>")
 *    }
 *  },
 *  hook = "url_outbound_alter"
 * )
 */
class HookURLOutboundAlter extends HookConverterBase {

  use ClassGeneratorTrait;

  /**
   * {@inheritdoc}
   */
  public function convert() {
    $service = [
      'class' => $this->module->getNS() . '\\OutboundPathProcessor',
      'tags' => [
        [ 'name' => 'path_processor_outbound' ],
      ],
    ];
    $this->module->addService('outbound_path_processor', $service);

    $processor = $this->generate();
    $this->addMethod($this->module->getHook('url_outbound_alter'), $processor, 'processOutbound');

    $this->module->write('src/OutboundPathProcessor.php', $processor->parents()[0]);
    drush_log('Generated outbound path processor in src/OutboundPathProcessor.php', 'success');
  }
  
  /**
   * {@inheritdoc}
   */
  public function generate() {
    $render = [
      '#theme' => 'dmu_outbound_path_processor',
      '#module' => $this->module->getMachineName(),
    ];

    return $this->parse($render);
  }

}
