<?php

/**
 * @file
 * Contains Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide\HookUninstall.
 */

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide;

use Drupal\drupalmoduleupgrader\Converter\HookConverterBase;
use Pharborist\Filter;

/**
 * @Converter(
 *  id = "HookUninstall",
 *  description = @Translation("Removes variable_del() calls from hook_uninstall()."),
 *  message = @Translation("Uninstalling default configuration is handled automatically by core."),
 *  hook = "uninstall"
 * )
 */
class HookUninstall extends HookConverterBase {

  /**
   * {@inheritdoc}
   */
  public function convert() {
    $calls = $this->module->getHook('uninstall')->find(Filter::isFunctionCall('variable_del'));

    if ($calls) {
      foreach ($calls as $call) {
        $call->remove();
      }

      if ($calls->count() > 0) {
        $count = \Drupal::translation()->formatPlural(sizeof($calls), '1 call', '@count calls');
        drush_log("Deleted {$count} to variable_del() from hook_uninstall() implementation.", 'success');
      }
    }
  }

}
