<?php

/**
 * @file
 * Contains Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide\HookWatchdog.
 */

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\ModuleWide;

use Drupal\drupalmoduleupgrader\Converter\HookConverterBase;
use Drupal\drupalmoduleupgrader\Utility\ClassGeneratorTrait;

/**
 * @Converter(
 *  id = "HookWatchdog",
 *  description = @Translation("Converts hook_watchdog() to an implementation of \\Psr\\Log\\LoggerInterface."),
 *  documentation = {
 *    {
 *      "url" = "https://www.drupal.org/node/2270941",
 *      "title" = @Translation("<code>hook_watchdog()</code> removed; <code>watchdog()</code> deprecated and replaced by PSR-3 compliant logging service")
 *    }
 *  },
 *  message = @Translation("<code>hook_watchdog()</code> has been removed."),
 *  hook = "watchdog"
 * )
 */
class HookWatchdog extends HookConverterBase {

  use ClassGeneratorTrait;

  /**
   * {@inheritdoc}
   */
  public function convert() {
    $service = [
      'class' => $this->module->getNS() . '\\Logger\\DefaultLogger',
      'tags' => [
        [ 'name' => 'logger' ],
      ],
    ];

    $this->module->addService('default_logger', $service);
    $this->module->write('src/Logger/DefaultLogger.php', $this->generate()->parents()[0]);

    drush_log('Generated default logger in src/Logger/DefaultLogger.php', 'success');
  }

  /***
   * {@inheritdoc}
   */
  public function generate() {
    $render = [
      '#theme' => 'dmu_logger',
      '#module' => $this->module->getMachineName(),
    ];

    return $this->parse($render);
  }

}
