<?php

/**
 * @file
 * Contains \Drupal\drupalmoduleupgrader\Utility\ConfigFormConverter.
 */

namespace Drupal\drupalmoduleupgrader\Utility;

use Pharborist\Filter;
use Pharborist\Parser;

/**
 * An extension of FormConverter that specifically converts configuration forms (i.e.,
 * forms that called the defunct system_settings_form()).
 */
class ConfigFormConverter extends FormConverter {

  /**
   * {@inheritdoc}
   */
  protected function getRenderable() {
    $render = parent::getRenderable();
    $render['#theme'] = 'dmu_config_form';

    return $render;
  }

  /**
   * {@inheritdoc}
   */
  public function generate() {
    // Change any call to system_settings_form() in the builder function to
    // parent::buildForm(). Just swap 'em right out.
    $calls = $this->builder->find(Filter::isFunctionCall('system_settings_form'));
    foreach ($calls as $call) {
      // @todo Use VariableNode::create() when it's added to Pharborist. @see #35
      // @todo $form_state should be whatever the actual local name of $form_state is
      $variable = Parser::parseSnippet('$form_state;');
      $variable->lastChild()->remove();

      $call->setName('parent::buildForm')->appendArgument($variable);
    }

    return parent::generate();
  }

  /**
   * {@inheritdoc}
   */
  protected function getHandler($type) {
    $handler = parent::getHandler($type);

    if ($handler && $type == 'submit') {
      // @todo Add a doc comment to the handler
      $handler->setName('_submitForm');
    }

    return $handler;
  }

}
