<?php

/**
 * @file \Drupal\drupalmoduleupgrader\Utility\ParametricRewriter\EntityPropertyAccessRewriter.
 */

namespace Drupal\drupalmoduleupgrader\Utility\ParametricRewriter;

use Pharborist\ObjectMethodCallNode;
use Pharborist\ObjectPropertyNode;
use Pharborist\Operator\AssignNode;

/**
 * Base class for parametric rewriters that modify get or set operations on entities.
 */
abstract class EntityPropertyAccessRewriter extends PropertyAccessRewriter {

  /**
   * The entity type definition of the parameter.
   *
   * @var \Drupal\Core\Entity\ContentEntityType
   */
  protected $entityInfo;

  /**
   * @var array
   */
  protected $propertyMap = [];

  /**
   * {@inheritdoc}
   */
  public function execute() {
    if (empty($this->entityInfo)) {
      throw new \RuntimeException("Cannot rewrite entity property accesses without an entity type definition.");
    }
    parent::execute();
  }

  protected function buildPropertyMap($entity_type) {
    $this->entityInfo = \Drupal::entityManager()->getDefinition($entity_type);

    $this->propertyMap = [
      $this->entityInfo->getKey('id') => 'id',
      $this->entityInfo->getKey('bundle') => 'bundle',
      $this->entityInfo->getKey('label') => 'label',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getter(ObjectPropertyNode $access) {
    $property = $access->getProperty()->__toString();

    if (isset($this->propertyMap[$property])) {
      $access->replaceWith(ObjectMethodCallNode::create($access->getObject(), $this->propertyMap[$property]));
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function setter(ObjectPropertyNode $access, AssignNode $assignment) {
    $property = $access->getProperty()->__toString();

    if ($property == $this->entityInfo->getKey('id') || $property == $this->entityInfo->getKey('bundle')) {
      // Never allow the entity ID or bundle to be set explicitly. That was
      // illegal in D7 anyway, but now it's strictly enforced.
      throw new \LogicException('Illegal attempt to set entity ID or bundle.');
    }
  }

}
